@@ -141,6 +141,7 @@ class FileStore(AbstractStore):
                 name=Experiment.DEFAULT_EXPERIMENT_NAME,
                 experiment_id=FileStore.DEFAULT_EXPERIMENT_ID,
                 artifact_uri=None,
+                tags=None,
             )
         # Create trash folder if needed
         if not exists(self.trash_folder):
@@ -272,7 +273,7 @@ class FileStore(AbstractStore):
         else:
             return PagedList(experiments, None)
 
-    def _create_experiment_with_id(self, name, experiment_id, artifact_uri):
+    def _create_experiment_with_id(self, name, experiment_id, artifact_uri, tags):
         artifact_uri = artifact_uri or append_to_uri_path(
             self.artifact_root_uri, str(experiment_id)
         )
@@ -284,6 +285,9 @@ class FileStore(AbstractStore):
         # As such, we should not include them in the meta file.
         del experiment_dict["tags"]
         write_yaml(meta_dir, FileStore.META_DATA_FILE_NAME, experiment_dict)
+        if tags is not None:
+            for tag in tags:
+                self.set_experiment_tag(experiment_id, tag)
         return experiment_id
 
     def _validate_experiment_name(self, name):
@@ -308,7 +312,7 @@ class FileStore(AbstractStore):
                     databricks_pb2.RESOURCE_ALREADY_EXISTS,
                 )
 
-    def create_experiment(self, name, artifact_location=None):
+    def create_experiment(self, name, artifact_location=None, tags=None):
         self._check_root_dir()
         self._validate_experiment_name(name)
         # Get all existing experiments and find the one with largest numerical ID.
@@ -319,7 +323,7 @@ class FileStore(AbstractStore):
             if e.experiment_id.isdigit()
         ]
         experiment_id = max(experiments_ids) + 1 if experiments_ids else 0
-        return self._create_experiment_with_id(name, str(experiment_id), artifact_location)
+        return self._create_experiment_with_id(name, str(experiment_id), artifact_location, tags)
 
     def _has_experiment(self, experiment_id):
         return self._get_experiment_path(experiment_id) is not None
